/*
 *  ncxmms
 *  Copyright 2003 Mariusz Gniazdowski
 *
 *  NCXmms is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  NCXmms is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include "defines.h"
#include "file.h"
#include "util.h"


/*
 * NAME:        write_log
 * DESCRIPTION: appends stuff to log 
 */

void write_log(long code, char *first, ...)
{
	va_list argp;
	char *path, *home;
	FILE *log;

	if (first == NULL)
		return;

	home = getenv("HOME");
	path = malloc(strlen(home) + 64);
	if (path == NULL)
		return;
	//FIXME: it should use ~/.ncxmms/
	sprintf(path, "%s/.ncxmms.log", home);

	log = fopen(path, "a");
	if (log != NULL) {
		if (code != 0)
			fprintf(log, "code: %d  ", (int) code);
		va_start(argp, first);
		vfprintf(log, first, argp);
		va_end(argp);
		fclose(log);
	}
	free(path);
} /* write_log() */

/*
 * NAME:        list_new_dir(path, out_dir_list, out_types, num_files, out_dir);
 * DESCRIPTION: Builds table of pointers for filenames in given dir.
 *		If operation fails, no changes are made to previorus dir_data.
 */
void list_new_dir(struct dir_data_s *dir_data, int file_abbrev_len, int dir_abbrev_len)
{

	DIR *dir;
	struct dirent *dir_entry;
	struct stat file_info;
	char *full_path;
	char **full_filenames,
		**abbrev_filenames,
		**full_dirnames,
		**abbrev_dirnames,
		**dir_contents,
		*file_types;
	int n, i, num_dirs, num_files, file_idx, dir_idx;

	dir = opendir(dir_data->path);
	if (dir == NULL) {
		write_log(0, "Cant open dir %s\n", dir_data->path);
		return;
	}
	/* Get number of all files */
	n=0;
	while (readdir(dir) != NULL)
		n++;

	/* Alloc memory for rows */
	dir_contents = (char **) malloc(sizeof(char *) * (n + 1));
	if (dir_contents == NULL) {
		/* If operation fails, no changes are made to previorus filelist */
		closedir(dir);
		return;
	}
	/* for free_2d_tab */
	dir_contents[n]=NULL;

	/* Alloc memory for filetype table */
	file_types = (char *) malloc(sizeof(char) * n);
	if (NULL == file_types) {
		free(dir_contents);
		closedir(dir);
		return; 
	}

	/* Alloc space and fill it with all filenames */
	rewinddir(dir);
	for (i = 0; i < n; i++) {
		dir_entry = readdir(dir);
		dir_contents[i] = (char *) malloc(strlen(dir_entry->d_name) + 1); /* XXX no check is made here... */
		strcpy(dir_contents[i], dir_entry->d_name);
	}
	closedir(dir);

	qsort(dir_contents, n, sizeof(char *), strcmp_wrap);

	/* Prepare buffer for path + filename */
	full_path = malloc(strlen(dir_data->path) + NAME_MAX + 2);
	if (NULL == full_path) {
		write_log(0, "Cant alloc memory for fullpath (used to check filetypes)\n");
		free_2d_tab((void **)dir_contents);
		free(file_types);
		return;
	}

	/* Get and save filetypes, and count number of dirs and files */
	num_dirs=0;
	num_files=0;
	for (i = 0; i < n; i++) {
		sprintf(full_path, "%s/%s", dir_data->path, dir_contents[i]);
		stat(full_path, &file_info);
		if (S_ISDIR(file_info.st_mode)) {
			file_types[i] = TYPE_DIR;
			num_dirs++;
		}
		else {
			file_types[i] = TYPE_FILE;
			num_files++;
		}
	}

	/* Alloc memory for full dir & filenames */
	full_filenames = (char **) malloc( sizeof(char *) * (num_files+1) );
	if(NULL==full_filenames) {
		free_2d_tab((void **)dir_contents);
		free(file_types);
		return;
	}
	full_filenames[num_files]=NULL;
	full_dirnames = (char **) malloc( sizeof(char *) * (num_dirs+1) );
	if(NULL==full_dirnames) {
		free(full_filenames); 
		free_2d_tab((void **)dir_contents);
		free(file_types);
		return;
	}
	full_dirnames[num_dirs]=NULL;


	/* According to file_types[] copy pointers 
	 * from dir_contents to proper table */
	dir_idx=0;
	file_idx=0;
	for(i=0; i<n; i++)
	{
		if(file_types[i]==TYPE_FILE) {
			full_filenames[file_idx]=dir_contents[i];
			file_idx++;
		} else {
			full_dirnames[dir_idx]=dir_contents[i];
			dir_idx++;
		}
	}

	/* Generate abbrevs for each table */
	if(!build_abbrev(full_filenames, &abbrev_filenames, num_files, file_abbrev_len)) {
		free(full_dirnames);
		free(full_filenames); 
		free_2d_tab((void **)dir_contents);
		free(file_types);
		return;
	}
	if(!build_abbrev(full_dirnames, &abbrev_dirnames, num_dirs,  dir_abbrev_len)) {
		free_2d_tab((void **)abbrev_filenames);
		free(full_dirnames);
		free(full_filenames); 
		free_2d_tab((void **)dir_contents);
		free(file_types);
		return;
	}
	
	/* After full success we can now change structure */
	if(dir_data->full_filenames)
		free_2d_tab((void **)dir_data->full_filenames);
	if(dir_data->abbrev_filenames)
		free_2d_tab((void **)dir_data->abbrev_filenames);
	if(dir_data->full_dirnames)
		free_2d_tab((void **)dir_data->full_dirnames);
	if(dir_data->abbrev_dirnames)
		free_2d_tab((void **)dir_data->abbrev_dirnames);

	dir_data->full_filenames = full_filenames;
	dir_data->abbrev_filenames = abbrev_filenames;
	dir_data->file_abbrev_len = file_abbrev_len;
	dir_data->num_files = num_files;
	dir_data->full_dirnames = full_dirnames;
	dir_data->abbrev_dirnames = abbrev_dirnames;
	dir_data->dir_abbrev_len = dir_abbrev_len;
	dir_data->num_dirs = num_dirs;
	free(dir_contents);
	free(file_types);
} /* list_new_dir() */

/*
 * NAME:        handle_newpath 
 * DESCRIPTION: Changes path after selecting new dir 
 */

int handle_newpath(char **path, const char *newdir)
{
	char *slash, *full_path;

	if (!strcmp(".", newdir))
		return 0;

	if (!strcmp("..", newdir)) {
		/*if act_path="/" do nothing */
		if (rindex(*path, '/') == index(*path, '/')) {
			if ((*path)[1] == '\0')
				return 0;
			else {
				(*path)[1] = '\0';
				return 1;
			}
		}
		slash = rindex(*path, '/');
		slash[0] = '\0';
	}
	else {
		/*otwarcie ciezki w przd */
		full_path = malloc(strlen(*path) + strlen(newdir) + 2);
		sprintf(full_path, "%s/%s", *path, newdir);
		free(*path);
		*path = full_path;
	}
	return 1;
} /* handle_newpath() */

